/*
Copyright (C) 2008 Chen Shih Sheng (shih.sheng@msa.hinet.net)

This file is part of PatentNet.

PatentNet is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PatentNet is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PatentNet.  If not, see <http://www.gnu.org/licenses/>.

*/


package PatentNet;

import java.io.*;
import java.net.*;
import java.util.*;


public class GooglePatent extends Patent {

	String raw_data;
	
	public static Date parseDate(String string){
		StringTokenizer st1 = new StringTokenizer(string,",");

		StringTokenizer st2 = new StringTokenizer(st1.nextToken());

		int month = parseMonth(st2.nextToken());
		int day = Integer.parseInt(st2.nextToken());
		int year =Integer.parseInt(st1.nextToken());
		return new Date(year,month,day);
	}

	public static int parseMonth(String string){
		int i = -1;

		if (string.equalsIgnoreCase("jan")) {
			i = 0;
		}
		if (string.equalsIgnoreCase("feb")) {
			i = 1;
		}
		if (string.equalsIgnoreCase("mar")) {
			i = 2;
		}
		if (string.equalsIgnoreCase("apr")) {
			i = 3;
		}
		if (string.equalsIgnoreCase("may")) {
			i = 4;
		}
		if (string.equalsIgnoreCase("jun")) {
			i = 5;
		}
		if (string.equalsIgnoreCase("jul")) {
			i = 6;
		}
		if(string.equalsIgnoreCase("aug")){
			i = 7;
		}
		if (string.equalsIgnoreCase("sep")) {
			i = 8;
		}
		if (string.equalsIgnoreCase("oct")) {
			i = 9;
		}
		if (string.equalsIgnoreCase("nov")) {
			i = 10;
		}
		if (string.equalsIgnoreCase("dec")) {
			i = 11;
		}

		return i;
		
	}

	/*
	public String parsePdfURL(){
		return parsePdfURL(raw_data);
	}

	public static String parsePdfURL(String string) {
		String s1 = "http://www.google.com/patents/download/";
		String s2 = "\"";
		return  s1+StringUtils.substring(string, s1, s2);
	}
	*/

	public String[] parseCitations(){
		return parseCitations(raw_data);
	}

	public static String[] parseCitations(String string) {

		String s1 = "<a name=\"patent_citations_anchor\">";
		String s2 = "</table>";
		string = StringUtils.substring(string, s1, s2);
		Vector vector = new Vector();

		if (string != null) {
			s1 = "/patents/";
			
			for (int i = string.indexOf(s1); i > -1; ) {
				string = string.substring(i+s1.length(), string.length());
				int j = string.indexOf(">");

				string = string.substring(j+1, string.length());
				int k = string.indexOf("<");

				vector.add(string.substring(0, k).trim());
				string = string.substring(j, string.length());
				i = string.indexOf(s1);
			}
		}

		String[] patent_numbers = new String[vector.size()];
		vector.toArray(patent_numbers);

		return patent_numbers;
	}



	public String[] parseReferencedBy() {
		return parseReferencedBy(raw_data);
	}


	public static String[] parseReferencedBy(String string) {
		String s1 = "<a name=\"patent_referenced_by_anchor\">";
		String s2 = "</table>";
		string = StringUtils.substring(string, s1, s2);
		Vector vector = new Vector();

		if (string != null) {
			s1 = "/patents/";

			for (int i = string.indexOf(s1); i > -1; ) {
				string = string.substring(i + s1.length(), string.length());
				int j = string.indexOf(">");

				string = string.substring(j + 1, string.length());
				int k = string.indexOf("<");

				vector.add(string.substring(0, k).trim());
				string = string.substring(j, string.length());
				i = string.indexOf(s1);
			}
		}



		String[] patent_numbers = new String[vector.size()];
		vector.toArray(patent_numbers);

		return patent_numbers;
	}





	public GooglePatent(File file) throws Exception {
		this(new FileWrapper(file).open());
	}
	


	public GooglePatent(String raw_data) throws Exception {
		this.raw_data = raw_data;
		parseALL();
	}

	public void parseALL() throws Exception{

		TTL = parseTTL(raw_data );
		
		ABST = parseABST(raw_data);
		PN = parsePN(raw_data);
		APD = parseAPD(raw_data);
		ISD = parseISD(raw_data);

		

		String Inventors = parseIN(raw_data);
		if (Inventors == null) {
			Inventors = parseINS(raw_data);
		}

		if (Inventors != null) {
			StringTokenizer tokenizer = new StringTokenizer(Inventors, ",");
			IN = new String[tokenizer.countTokens()];
			for (int i = 0; i < IN.length; i++) {
				IN[i] = tokenizer.nextToken().trim();
			}
		}

		

		AN = parseAN(raw_data);
		if (AN == null) {
			AN = parseANS(raw_data);
		}

		

		ACLM = parseACLM(raw_data);
		Citations = parseCitations(raw_data);
		ReferencedBy = parseReferencedBy(raw_data);
	}

	public static String parseTTL(String string) {
		String s1 = "<meta name=\"title\" content=\"";
		String s2 = "\"/>";

		String title = StringUtils.substring(string, s1, s2);
		if (title != null) {
			title = title.substring(title.indexOf("-") + 1);
		}

		return title;
	}

	
	public static String parseABST(String string) {
		String s1 = "<p class=\"patent_abstract_text\">";
		String s2 = "</p>";
		return URLConnectionWrapper.deTag(StringUtils.substring(string, s1, s2));
	}


	public static String parsePN(String string) {
		String s1 = "<title>";
		String s2 = "</title>";
		String title = StringUtils.substring(string, s1, s2);

		String PN = null;

		if (title != null) {
			StringTokenizer tokenizer = new StringTokenizer(title);

			

			while (tokenizer.hasMoreTokens()) {
				String token = (String)tokenizer.nextToken();
				if (token.equalsIgnoreCase("patent")) {
					PN = (String)tokenizer.nextToken();
					break;
				}
			}
		}



		return PN;
	}
	
	/*
	public static String parsePN(String string) {
		String s1 = "<b>Patent number</b>:";
		String s2 = "<br>";
		String PN = StringUtils.substring(string, s1, s2);
		if(PN!=null){
			PN = "US" + PN;
		}

		return PN;
	}
	 */

	public static String parseAPD(String string) {
		String s1 = "<b>Filing date</b>:";
		String s2 = "<br>";
		return StringUtils.substring(string, s1, s2);
	}

	public static String parseISD(String string) {
		String s1 = "<br><b>Issue date</b>:";
		String s2 = "<br>";
		return StringUtils.substring(string, s1, s2);
	}

	public static String parseIN(String string) {
		String s1 = "<b>Inventor</b>:";
		String s2 = "<br>";
		return URLConnectionWrapper.deTag_(StringUtils.substring(string, s1, s2));
	}

	public static String parseINS(String string) {
		String s1 = "<b>Inventors</b>:";
		String s2 = "<br>";
		return URLConnectionWrapper.deTag_(StringUtils.substring(string, s1, s2));
		
	}

	public static String parseAN(String string) {
		String s1 = "<b>Original Assignee</b>:";
		String s2 = "<br>";
		return URLConnectionWrapper.deTag_(StringUtils.substring(string, s1, s2));
	}

	public static String parseANS(String string) {
		String s1 = "<b>Original Assignees</b>:";
		String s2 = "<br>";
		return URLConnectionWrapper.deTag_(StringUtils.substring(string, s1, s2));
	}

	public static String parseACLM(String string) {
		String s1 = "<div id=patent_claims_v>";
		String s2 = "<a name=\"selected_pages_anchor\">";
		return URLConnectionWrapper.deTag(StringUtils.substring(string, s1, s2));
	}

	

	
	public static boolean isGooglePatent(String raw_data) {
		boolean b = false;
		if (raw_data != null) {
			if (parsePN(raw_data) != null) {
				b = true;
			}
		}
		return b;
	}

}
