/*
Copyright (C) 2008 Chen Shih Sheng (shih.sheng@msa.hinet.net)

This file is part of PatentNet.

PatentNet is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PatentNet is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PatentNet.  If not, see <http://www.gnu.org/licenses/>.

*/

package PatentNet;

import java.io.*;
import java.net.*;
import java.util.*;


public class GooglePatentGrabber extends PatentGrabber implements Runnable {

	String search_string;
	File put_folder;

	
	int FC_depth;
	int BC_depth;

	boolean grabPdf;
	boolean grabFamily;


	public GooglePatentGrabber(String search_string, File put_folder,
		 int FC_depth, int BC_depth, boolean grabPdf, boolean grabFamily)throws Exception {


		this.search_string = search_string;
		this.put_folder = put_folder;

	
		this.FC_depth = FC_depth;
		this.BC_depth = BC_depth;

		this.grabPdf = grabPdf;
		this.grabFamily = grabFamily;

		initCookie();

	}



	public String search(String string)throws Exception {
		return getResponseString("http://www.google.com/patents?q=" + URLEncoder.encode(string));
	}

	public void initCookie()throws Exception {
		URL url = new URL("http://www.google.com");
		HttpURLConnection urlConn = (HttpURLConnection)url.openConnection();
		//urlConn.setRequestProperty("User-Agent", "Mozilla/4.0 (compatible; MSIE 6.0; Windows 2000)");
		cookie = urlConn.getHeaderField("Set-Cookie");
	}


	public String fix_url(String url) {
		StringBuffer buffer = new StringBuffer();

		for (int i = 0; i < url.length(); i++) {
			char c = url.charAt(i);
			switch (c) {
				case '\"':
				case '\'':
					buffer.append(" ");
					break;
				case '&':
					String tmp = url.substring(i, i + 5);
					if (tmp.equalsIgnoreCase("&amp;")) {

						i += 4;
					}
					buffer.append(c);
					break;
				default:
					buffer.append(c);
			}
		}

		StringTokenizer tokenizer = new StringTokenizer(buffer.toString());

		while (tokenizer.hasMoreTokens()) {
			url = tokenizer.nextToken();
			//System.out.println(url);
			if (url.toLowerCase().indexOf("/search?") > -1) {
				break;
			}
		}

		return url;
	}


	//this is a filter for public static String[] parse_pages(String content)
	public static String parse_start(String url) {
		String start_value = "";

		int i = url.indexOf("start=");

		if (i > -1) {
			String tmp = url.substring(i, url.length());
			int j = tmp.indexOf("&");
			start_value = tmp.substring(0, j);
		}

		return start_value;
	}


	//returns all page links in a search result 
	//e.g.  Goooooooooooooooooogle
	//      1 2 3 4 5 6 7 8 Next
	public String[] parse_pages(String content)throws Exception {

		Vector vector = new Vector();
		int j = content.indexOf("/search?");
		while (j > -1) {
			content = content.substring(j, content.length());
			int i = content.indexOf("/search?");
			int k = content.indexOf(">");
			String url = content.substring(i, k);
			url = fix_url(url);

			//filter
			//url with &start= is a page
			String start_value = parse_start(url);
			if (start_value != null) {
				vector.add("http://www.google.com" + url);
			}

			content = content.substring(k);
			j = content.indexOf("/search?");
		}

		String[] urls = new String[vector.size()];
		vector.toArray(urls);

		return urls;
	}


	//collects all page links in all pages
	//since the links are cross referenced 
	//the duplicates are removed through a hashtable check
	public void parse_all_pages(String url0, Hashtable table)throws Exception {

		String start_value = parse_start(url0);

		if (table.get(start_value) == null) {
			String content = getResponseString(url0);
			String[] urls = parse_pages(content);
			table.put(start_value, content);
			for (int i = 0; i < urls.length; i++) {
				parse_all_pages(urls[i], table);
			}
		}


	}

	/*
	public String[] parse_patents2(String content)throws Exception {

		Vector vector = new Vector();
		int j = content.indexOf("/patents?id");

		while (j > -1) {
			content = content.substring(j, content.length());
			int i = content.indexOf("/patents?id");
			int k = content.indexOf(">");
			String url = content.substring(i, k);
			url = fix_url(url);

			//filter
			//url without printsec= is an overview
			if (url.indexOf("printsec=") < 0) {
				vector.add("http://www.google.com" + url);
			}

			content = content.substring(k);
			j = content.indexOf("/patents?id");

		}

		String[] urls = new String[vector.size()];
		vector.toArray(urls);

		return urls;
	}
	*/

	public String[] parse_patents(String content)throws Exception {

		Vector vector = new Vector();

		String key = "/patents/";
		int j = content.indexOf(key);

		while (j > -1) {
			content = content.substring(j + key.length(), content.length());
			int k = content.indexOf("<");
			String patent_number = content.substring(0, k).trim();

			vector.add(patent_number);

			content = content.substring(k);
			j = content.indexOf(key);
		}

		String[] urls = new String[vector.size()];
		vector.toArray(urls);

		return urls;
	}


	public void grab_patent_and_bcs(String patent_number, Hashtable table, int depth)throws Exception {



		GooglePatent patent = grab_patent(patent_number, table);

		if (patent != null) {

			depth++;

			if (depth <= BC_depth) {

				String[] patent_numbers = patent.getCitations();

				for (int i = 0; i < patent_numbers.length; i++) {
					grab_patent_and_bcs(patent_numbers[i], table, depth);
				}
			}
		}



		depth--;
	}


	public void grab_patent_and_fcs(String patent_number, Hashtable table, int depth)throws Exception {



		GooglePatent patent = grab_patent(patent_number, table);

		if (patent != null) {

			depth++;

			if (depth <= FC_depth) {

				String[] patent_numbers = patent.getReferencedBy();

				for (int i = 0; i < patent_numbers.length; i++) {
					grab_patent_and_fcs(patent_numbers[i], table, depth);
				}
			}
		}


		depth--;
	}


	public void grab_patent_pdf(String patent_number)throws Exception {


		String url = "http://www.google.com/patents/" + patent_number+".pdf";

		byte[] data = null;

		while (data == null) {
			data = getResponseBytes(url);
			if (new String(data).indexOf("403 Forbidden") > -1) {
				data = null;
				Thread.sleep(10000);
			}
		}


		new FileWrapper(new File(put_folder, patent_number + ".pdf")).save(data);
		Thread.sleep(10000);
	}

	public GooglePatent grab_patent(String patent_number, Hashtable table)throws Exception {

		GooglePatent patent = null;

		if (table.get(patent_number) == null) {

			String url = "http://www.google.com/patents/" + patent_number;


			String rawdata = null;

			while (rawdata == null) {
				rawdata = getResponseString(url);
				
				if (rawdata.indexOf("403 Forbidden") > -1) {
					rawdata = null;
					Thread.sleep(10000);
				}
			}

			if (GooglePatent.isGooglePatent(rawdata)) {
				patent = new GooglePatent(rawdata);
				table.put(patent_number, patent);
				new FileWrapper(new File(put_folder, patent.getPN() + ".htm")).save(rawdata);

	

			}
		} else {
			patent = (GooglePatent)table.get(patent_number);
		}

		return patent;

	}


	public void run() {
		try {

			Hashtable table_pages = new Hashtable();

			parse_all_pages("http://www.google.com/patents?q=" + URLEncoder.encode(search_string), table_pages);

			Enumeration enu_page = table_pages.elements();


			Hashtable table_patents = new Hashtable();

			while (enu_page.hasMoreElements()) {

				String[] patent_numbers = parse_patents((String)enu_page.nextElement());

				for (int i = 0; i < patent_numbers.length; i++) {
					grab_patent_and_bcs(patent_numbers[i], table_patents, 0);
					grab_patent_and_fcs(patent_numbers[i], table_patents, 0);
				}
	

			}

			if (grabFamily) {
				espPatentGrabber grabber = new espPatentGrabber();
				Enumeration enu_patent = table_patents.elements();
				while (enu_patent.hasMoreElements()) {
					Patent patent = (Patent)enu_patent.nextElement();
					grabber.grabFamily(put_folder, patent.getPN());
				}
			}

			if (grabPdf) {
				Enumeration enu_patent = table_patents.elements();
				while (enu_patent.hasMoreElements()) {
					GooglePatent patent = (GooglePatent)enu_patent.nextElement();

					grab_patent_pdf(patent.getPN());
				
				}
			}

		

			System.out.println(search_string + "...done");


		} catch (Exception e) {
			e.printStackTrace();
		}
	}






}
