/*
 * @(#)Graph.java	1.11 01/12/03
 *
 * Copyright 2002 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package PatentNet;

import java.util.*;
import java.awt.*;
import java.awt.event.*;

class Node {

	double x;
	double y;

	boolean fixed;

	String lbl;
}


class Edge {
	int from;
	int to;

	int weight;

	double len;
}


class GraphPanel extends Panel implements Runnable, MouseListener, MouseMotionListener {

	int nnodes;
	Node nodes[] = new Node[10000];

	int nedges;
	Edge edges[] = new Edge[10000];

	Thread relaxer;

	boolean relax = true;
	boolean weight = true;
	boolean force_center = true;
	boolean wall = true;
	boolean spring = true;
	boolean repulsion = true;
	boolean gravity = false;


	GraphPanel() {
		addMouseListener(this);
	}

	int findNode(String lbl) {
		for (int i = 0; i < nnodes; i++) {
			if (nodes[i].lbl.equalsIgnoreCase(lbl)) {
				return i;
			}
		}
		return -1;
	}

	Edge findEdge(int from, int to) {
		for (int i = 0; i < nedges; i++) {
			if ((edges[i].from == from && edges[i].to == to)) {
				return edges[i];
			}
		}
		return null;
	}

	int addNode(String lbl) {
		int i = findNode(lbl);
		if (i < 0) {
			Node n = new Node();
			n.x = 1000 * Math.random();
			n.y = 1000 * Math.random();
			n.lbl = lbl;
			nodes[nnodes] = n;
			return nnodes++;
		}
		else {
			return i;
		}
	}

	void addEdge(String from, String to) {
		addEdge(from, to, 100);
	}
	void addEdge(String from, String to, int len) {

		int i = findNode(from);
		int j = findNode(to);

		if (i > -1 && j > -1 && i != j) {
			Edge e = findEdge(i, j);
			if (e != null) {
				e.weight++;
			}
			else {
				e = new Edge();
				e.from = findNode(from);
				e.to = findNode(to);
				e.len = len;
				e.weight = 1;
				edges[nedges++] = e;
			}
		}

	}

	public void run() {
		Thread me = Thread.currentThread();
		while (relaxer == me) {

			if (relax) {
				relax();
			}

			repaint();

			try {
				Thread.sleep(10);
			}
			catch (InterruptedException e) {
				break;
			}
		}
	}

	synchronized void relax() {

		
		Dimension d = getSize();
		double xc = d.width / 2;
		double yc = d.height / 2;


		
		for (int i = 0; i < nnodes; i++) {
			Node n = nodes[i];

			
			if (wall) {

				
				if (n.x < 10) {
					n.x = 10;
				}
				if (n.x > d.width - 10) {
					n.x = d.width - 10;
				}

				if (n.y < 10) {
					n.y = 10;
				}
				if (n.y > d.height - 10) {
					n.y = d.height - 10; ;
				}

			}

			
			if (force_center) {
				double vx = n.x - xc;
				double vy = n.y - yc;

				
				double len = Math.sqrt(vx * vx + vy * vy);
				
				
				double k = 0.0001;

				
				double force = len * k;
				
				
				double dx = force * vx;
				double dy = force * vy;

				if (pick != nodes[i]) {
					n.x -= dx;
					n.y -= dy;
				}
			}

			
			if (gravity){

				n.y +=1;


			}
		}


		
		for (int i = 0; i < nnodes; i++) {
			Node n1 = nodes[i];

			for (int j = 0; j < nnodes; j++) {
				
				
				if (i == j) {
					continue;
				}

				Node n2 = nodes[j];
				double vx = n1.x - n2.x;
				double vy = n1.y - n2.y;

				
				double len = Math.sqrt(vx * vx + vy * vy);

				
				if(len<1){
					len = 1;
				}

				double force = 0;

				
				Edge edge = findEdge(i, j);

				
				if (edge != null) {

					
					if(spring){
						double k=0.001;
						force = (len - edge.len) * k;
					}

				
				}
				
				else {

					
					if (repulsion){
						force =- 10/(len*len);
					}
				}

				
				double dx = force * vx;
				double dy = force * vy;

				
				if (pick != n1) {
					n1.x -= dx;
					n1.y -= dy;
				}

				if (pick != n2) {
					n2.x += dx;
					n2.y += dy;
				}


			}

		}

	}

	Node pick;
	boolean pickfixed;
	Image offscreen;
	Dimension offscreensize;
	Graphics offgraphics;

	final Color fixedColor = Color.red;
	final Color selectColor = Color.pink;
	final Color edgeColor = Color.black;
	final Color nodeColor = new Color(250, 220, 100);
	final Color stressColor = Color.red;
	final Color arcColor1 = Color.black;
	final Color arcColor2 = Color.pink;
	final Color arcColor3 = Color.red;

	public void paintNode(Graphics g, Node n, FontMetrics fm) {
		int x = (int)n.x;
		int y = (int)n.y;
		g.setColor((n == pick) ? selectColor : (n.fixed ? fixedColor : nodeColor));
		int w = fm.stringWidth(n.lbl) + 10;
		int h = fm.getHeight() + 4;
		g.fillRect(x - w / 2, y - h / 2, w, h);
		g.setColor(Color.black);
		g.drawRect(x - w / 2, y - h / 2, w - 1, h - 1);
		g.drawString(n.lbl, x - (w - 10) / 2, (y - (h - 4) / 2) + fm.getAscent());
	}

	public synchronized void update(Graphics g) {
		Dimension d = getSize();
		if ((offscreen == null) || (d.width != offscreensize.width) || (d.height != offscreensize.height)) {
			offscreen = createImage(d.width, d.height);
			offscreensize = d;
			if (offgraphics != null) {
				offgraphics.dispose();
			}
			offgraphics = offscreen.getGraphics();
			offgraphics.setFont(getFont());
		}

		offgraphics.setColor(getBackground());
		offgraphics.fillRect(0, 0, d.width, d.height);


		for (int i = 0; i < nedges; i++) {
			Edge e = edges[i];
			int x1 = (int)nodes[e.from].x;
			int y1 = (int)nodes[e.from].y;
			int x2 = (int)nodes[e.to].x;
			int y2 = (int)nodes[e.to].y;
			double lenn = Math.sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2));
			int len = (int)Math.abs(lenn - e.len);


			if (pick != nodes[e.from] && pick != nodes[e.to]) {
				offgraphics.setColor(edgeColor);
			}
			else {
				offgraphics.setColor(Color.blue);
			}


			offgraphics.drawLine(x1, y1, x2, y2);

			double dx = 15 * (x1 - x2) / lenn;
			double dy = 15 * (y1 - y2) / lenn;


			int[] x = new int[3];
			int[] y = new int[3];

			x[0] = (int)(x2 + (x1 - x2) / 3);
			y[0] = (int)(y2 + (y1 - y2) / 3);

			double cos = Math.cos(Math.PI / 12);
			double sin = Math.sin(Math.PI / 12);

			x[1] = (int)(x[0] + dx * cos - dy * sin);
			y[1] = (int)(y[0] + dx * sin + dy * cos);
			x[2] = (int)(x[0] + dx * cos + dy * sin);
			y[2] = (int)(y[0] - dx * sin + dy * cos);



			offgraphics.fillPolygon(new Polygon(x, y, 3));

		}

		if (weight) {

			for (int i = 0; i < nedges; i++) {
				Edge e = edges[i];
				int x1 = (int)nodes[e.from].x;
				int y1 = (int)nodes[e.from].y;
				int x2 = (int)nodes[e.to].x;
				int y2 = (int)nodes[e.to].y;

				String lbl = String.valueOf(e.weight);

				if (pick != nodes[e.from] && pick != nodes[e.to]) {
					offgraphics.setColor(Color.pink);
				}
				else {
					offgraphics.setColor(Color.red);
				}
				offgraphics.drawString(lbl, x2 + (x1 - x2) / 3, y2 + (y1 - y2) / 3);
				offgraphics.setColor(edgeColor);
			}

		}



		FontMetrics fm = offgraphics.getFontMetrics();
		for (int i = 0; i < nnodes; i++) {
			paintNode(offgraphics, nodes[i], fm);
		}

		g.drawImage(offscreen, 0, 0, null);
	}

	//1.1 event handling
	public void mouseClicked(MouseEvent e) {
	}

	public void mousePressed(MouseEvent e) {
		addMouseMotionListener(this);
		double bestdist = Double.MAX_VALUE;
		int x = e.getX();
		int y = e.getY();
		for (int i = 0; i < nnodes; i++) {
			Node n = nodes[i];
			double dist = (n.x - x) * (n.x - x) + (n.y - y) * (n.y - y);
			if (dist < bestdist) {
				pick = n;
				bestdist = dist;
			}
		}
		pickfixed = pick.fixed;
		pick.fixed = true;
		pick.x = x;
		pick.y = y;
		repaint();
		e.consume();
	}

	public void mouseReleased(MouseEvent e) {
		removeMouseMotionListener(this);
		if (pick != null) {
			pick.x = e.getX();
			pick.y = e.getY();
			pick.fixed = pickfixed;
			//pick = null;
		}
		repaint();
		e.consume();
	}

	public void mouseEntered(MouseEvent e) {
	}

	public void mouseExited(MouseEvent e) {
	}

	public void mouseDragged(MouseEvent e) {
		pick.x = e.getX();
		pick.y = e.getY();
		repaint();
		e.consume();
	}

	public void mouseMoved(MouseEvent e) {
	}

	public void start() {
		relaxer = new Thread(this);
		relaxer.start();
	}

	public void stop() {
		relaxer = null;
	}

}
