/*
Copyright (C) 2008 Chen Shih Sheng (shih.sheng@msa.hinet.net)

This file is part of PatentNet.

PatentNet is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PatentNet is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PatentNet.  If not, see <http://www.gnu.org/licenses/>.

*/
package PatentNet;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;


public class PatentAnalyzer {

	File dir;
	Patent[] PATENT;


	public double averageReferencedBy() {
		double sum = 0;
		for (int i = 0; i < PATENT.length; i++) {
			sum += PATENT[i].ReferencedBy.length;
		}
		return sum / PATENT.length;
	}

	public double deviationReferencedBy() {
		double average = averageReferencedBy();
		double sum = 0;
		for (int i = 0; i < PATENT.length; i++) {
			sum += Math.pow(PATENT[i].ReferencedBy.length - average, 2);
		}
		sum /= PATENT.length - 1;
		return Math.sqrt(sum);
	}

	public Patent findPatent(String PN) {
		Patent patent = null;
		for (int i = 0; i < PATENT.length; i++) {
			if (PATENT[i].getPN().equals(PN)) {
				patent = PATENT[i];
				break;
			}
		}
		return patent;
	}

	public String sortFamily()throws Exception {

		Vector vector = new Vector();
		
		for (int i = 0; i < PATENT.length; i++) {
			FileWrapper filewrapper = new FileWrapper(new File(dir, PATENT[i].getPN() + "_family.htm"));
			if (filewrapper.file.exists()) {
				vector.add(new Countee(PATENT[i].getPN(), espPatentGrabber.parseFamily(filewrapper.open())));
			}
		}

		Countee[] C = new Countee[vector.size()];
		vector.toArray(C);
		Arrays.sort(C);
		return toString(C);
	}

	public String sortReferencedBy() {
		Countee[] C = new Countee[PATENT.length];
		for (int i = 0; i < C.length; i++) {
			C[i] = new Countee(PATENT[i].getPN(), PATENT[i].ReferencedBy.length);
		}
		Arrays.sort(C);

		return toString(C);
	}

	public String toString(Object[] OBJECT) {
		StringBuffer buffer = new StringBuffer();
		for (int i = 0; i < OBJECT.length; i++) {
			buffer.append(OBJECT[i].toString());
			buffer.append("\n");
		}
		return buffer.toString();
	}

	public String sortKeywords() {
		String[] Articles = new String[PATENT.length];
		for (int i = 0; i < Articles.length; i++) {
			Articles[i] = PATENT[i].getTTL() + " " + PATENT[i].getABST();
		}

		Countee[] C = KeywordCounter.count(Articles);

		return toString(C);
	}

	public void genCitationsNetworkIN() {
		GraphPanel panel = new GraphPanel();

		for (int i = 0; i < PATENT.length; i++) {
			for (int j = 0; j < PATENT[i].IN.length; j++) {
				panel.addNode(PATENT[i].IN[j]);
			}
		}

		for (int i = 0; i < PATENT.length; i++) {
			for (int j = 0; j < PATENT[i].Citations.length; j++) {
				Patent patent = findPatent(PATENT[i].Citations[j]);
				if (patent != null) {

					for (int k = 0; k < PATENT[i].IN.length; k++) {

						for (int m = 0; m < patent.IN.length; m++) {
							panel.addEdge(PATENT[i].IN[k], patent.IN[m]);

						}

					}

				}
			}
		}

		new GraphFrame(dir.getName(), panel);

	}


	public void genCitationsNetworkAN() {
		GraphPanel panel = new GraphPanel();

		for (int i = 0; i < PATENT.length; i++) {
			panel.addNode(PATENT[i].getAN());
		}

		for (int i = 0; i < PATENT.length; i++) {
			for (int j = 0; j < PATENT[i].Citations.length; j++) {

				Patent patent = findPatent(PATENT[i].Citations[j]);

				if (patent != null) {
					panel.addEdge(patent.getAN(), PATENT[i].getAN());
				}

			}
		}

		new GraphFrame(dir.getName(), panel);
	}


	public void genCitationsNetworkPN() {
		GraphPanel panel = new GraphPanel();

		for (int i = 0; i < PATENT.length; i++) {
			panel.addNode(PATENT[i].PN);
		}

		for (int i = 0; i < PATENT.length; i++) {

			for (int j = 0; j < PATENT[i].Citations.length; j++) {
				panel.addEdge(PATENT[i].PN, PATENT[i].Citations[j]);
			}

		}

		new GraphFrame(dir.getName(), panel);
	}

	public PatentAnalyzer(File dir)throws Exception {
		this.dir = dir;
		PATENT = loadPatents(dir);
	}

	public String toString() {
		StringBuffer buffer = new StringBuffer();

		for (int i = 0; i < PATENT.length; i++) {
			buffer.append(PATENT[i].toString());
			buffer.append("\n");
		}

		return buffer.toString();

	}


	public String toEndNote() {
		StringBuffer buffer = new StringBuffer();

		buffer.append("FN ISI Export Format\n");
		buffer.append("VR 1.0\n");

		for (int i = 0; i < PATENT.length; i++) {

			if (PATENT[i].getISD() != null) {

				buffer.append("PT Patent\n");

				buffer.append("PN " + PATENT[i].getPN() + "\n");



				String[] names = PATENT[i].getINs();
				if (names != null) {
					if (names.length > 0) {
						for (int j = 0; j < names.length; j++) {

							StringTokenizer tokenizer = new StringTokenizer(names[j]);

							int p = 0;
							int q = tokenizer.countTokens();
							StringBuffer firstname = new StringBuffer();

							String surename = "";
							while (tokenizer.hasMoreTokens()) {
								p++;
								if (p >= q) {
									surename = tokenizer.nextToken();
								} else {
									firstname.append(" " + tokenizer.nextToken());
								}
							}


							buffer.append("AU " + surename + ", " + firstname.toString().trim() + "\n");
						}
					}
				}


				buffer.append("TI " + PATENT[i].getTTL() + "\n");

				buffer.append("AB " + PATENT[i].getABST() + "\n");

				buffer.append("AE " + PATENT[i].getAN() + "\n");



				
				String[] citations = PATENT[i].getCitations();
				if (citations != null) {
					if (citations.length > 0) {
						
						for (int j = 0; j < citations.length; j++) {
							buffer.append("CP "+citations[j] + "\n");
						}
					}
				}
				

				String APD = PATENT[i].getAPD();

				if (APD != null) {
					StringTokenizer tokenizer = new StringTokenizer(APD, ",");

					//buffer.append("ISD " + ISD + "\n");

					if (tokenizer.countTokens() > 1) {
						String pd = tokenizer.nextToken().trim();
						buffer.append("PD " + pd + "\n");
						String py = tokenizer.nextToken().trim();
						buffer.append("PY " + py + "\n");
					}
				}
				buffer.append("ER\n");
			}

			
		}



		buffer.append("EF\n");

		return buffer.toString();

	}

	public static Patent[] loadPatents(File dir)throws Exception {
		Vector vector = new Vector();
		String[] list = dir.list();
		for (int i = 0; i < list.length; i++) {
			File file = new File(dir, list[i]);
			FileWrapper filewrapper = new FileWrapper(file);

			if (file.isFile() && filewrapper.getEXT().equalsIgnoreCase("htm")) {
				String raw_data = filewrapper.open();

				Patent patent = PatentFactory.parsePatent(raw_data);

				if (patent != null) {
					vector.add(patent);
				}
			}
		}
		Patent[] PATENT = new Patent[vector.size()];
		vector.toArray(PATENT);

		return PATENT;

	}


}
